<?php

namespace wordefinery;

final class LiveinternetCounter {

    const VERSION = '0.8.0';
    const DB = false;
    private $path = '';
    private $_is_counter = 0;

    private $size_idx  = array(1=>'88x31', 2=>'88x15', 3=>'31x31', 4=>'88x120');
    private $style_idx = array(1=>array(52, 53, 54, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 57, 58),
                               2=>array(23, 24, 25, 26),
                               3=>array(38, 39, 40, 41, 42, 43, 44, 45, 50),
                               4=>array(27, 28, 29));
    private $color_idx = array(1=>18, 2=>18, 3=>18, 4=>20);

    function __construct($path) {
        $this->path = $path;

        $this->plugin_title = __('Wordefinery LiveInternet Counter');
        $this->plugin_slug = 'wordefinery-liveinternetcounter';

        $o = \Wordefinery::Option($this->plugin_slug);
        if (isset($o) && is_array($o)) {
            $this->size = $o['size'];
            $this->style = $o['style'];
            $this->color = $o['color'];
            $this->align = $o['align'];
            $this->mode = $o['mode'];
            $this->page_title = $o['page_title'];
        } else {
            $this->size = key($this->size_idx);
            $this->style = current($this->style_idx[$this->size]);
            $this->color = 1;
            $this->align = 'center';
            $this->mode = 'widget';
            $this->page_title = '0';
        }
        list($this->width, $this->height) = explode('x', $this->size_idx[$this->size]);

        \add_action('admin_menu', array(&$this, 'AdminMenu'));
        \add_action('admin_init', array(&$this, 'AdminInit'));
        \load_plugin_textdomain( $this->plugin_slug, false, $this->path . DIRECTORY_SEPARATOR . 'languages' . DIRECTORY_SEPARATOR );
        \wp_register_style($this->plugin_slug.'-settings', WP_PLUGIN_URL . '/' . $this->path . '/(css)/settings_page.css', array(), self::VERSION );
        \wp_register_script($this->plugin_slug.'-settings', WP_PLUGIN_URL . '/' . $this->path . '/(js)/settings_page.js', array('jquery'), self::VERSION );

        switch ($this->mode) {
            case 'widget':
                \add_action('widgets_init', function () { return register_widget('\\wordefinery\\LiveinternetCounterWidget'); });
                break;
            case 'footer':
                \add_action('wp_footer', array(&$this, 'Footer'));
                break;
            case 'shortcode':
                \add_shortcode( 'liveinternetcounter', array(&$this, 'Shortcode'));
                break;
        }
        \add_filter('wp_nav_menu', array(&$this, 'Counter'));
        \add_filter('wp_page_menu', array(&$this, 'Counter'));

    }

    function AdminInit() {
        \register_setting( $this->plugin_slug, 'wordefinery', array(&$this, 'SettingsValidate') );
    }

    function AdminMenu() {
        $page = \add_options_page(
            __('Settings') . ' &mdash; ' . $this->plugin_title,
            __('LiveInternet Counter'),
            'manage_options',
            $this->plugin_slug . '-settings',
            array(&$this, 'SettingsPage')
        );

        $slug = $this->plugin_slug;
        \add_action( 'admin_print_styles-' . $page, function () use ($slug) { \wp_enqueue_style($slug.'-settings'); } );
        \add_action( 'admin_print_scripts-' . $page, function () use ($slug) { \wp_enqueue_script($slug.'-settings'); } );
        // \add_action("load-$page", array( &$this, 'help_tabs'));
    }

    function SettingsPage() {
       include(WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . $this->path . DIRECTORY_SEPARATOR . 'inc' . DIRECTORY_SEPARATOR . 'settings_page.php');
    }

    function SettingsValidate($v) {
        if (!isset($this->size_idx[$v['size']])) $v['size'] = key($this->size_idx);
        if (!isset($v['style'])) {
            $v['style'] = current($this->style_idx[$v['size']]);
        } else {
            $st_idx = array();
            foreach($this->size_idx as $i=>$s) $st_idx += array_fill_keys($this->style_idx[$i], $i);
            if (!isset($st_idx[$v['style']])) {
                $v['style'] = current($this->style_idx[$v['size']]);
            } elseif (isset($st_idx[$v['style']]) && $v['size']!=$st_idx[$v['style']]) {
                $v['size'] = $st_idx[$v['style']];
            }
        }
        if (!isset($v['color'])) {
            $v['color'] = 1;
        } elseif ($v['color'] > $this->color_idx[$v['size']] || $v['color'] < 1) {
            $v['color'] = 1;
        }
        if (!isset($v['align'])) {
            $v['align'] = 'center';
        } elseif ($v['align'] != 'center' && $v['align'] != 'left' && $v['align'] != 'right') {
            $v['align'] = 'center';
        }
        if (!isset($v['mode'])) {
            $v['mode'] = 'widget';
        } elseif ($v['mode'] != 'widget' && $v['mode'] != 'shortcode' && $v['mode'] != 'footer') {
            $v['mode'] = 'widget';
        }
        return $v;
    }

    function Shortcode($args) {
        static $x = 0;
        if ($x) return;
        $x = 1;
        return $this->Counter().
<<<END
<!--LiveInternet logo--><a href="http://www.liveinternet.ru/click"
target="_blank"><img src="http://counter.yadro.ru/logo?{$this->style}.{$this->color}" title="LiveInternet Counter"
alt="" border="0" width="{$this->width}" height="{$this->height}"/></a><!--/LiveInternet-->
END;
    }

    function Footer() {
        echo $this->Counter().
<<<END
<div style="text-align:{$this->align}"><!--LiveInternet logo--><a href="http://www.liveinternet.ru/click"
target="_blank"><img src="http://counter.yadro.ru/logo?{$this->style}.{$this->color}" title="LiveInternet Counter"
alt="" border="0" width="{$this->width}" height="{$this->height}"/></a><!--/LiveInternet--></div>
END;
    }

    function Counter($nav_menu = '') {
        if ($this->_is_counter) return $nav_menu;
        $this->_is_counter = 1;
        if ($this->page_title) {
            return $nav_menu.
<<<END
<!--LiveInternet counter--><script type="text/javascript"><!--
new Image().src = "//counter.yadro.ru/hit?r"+
escape(document.referrer)+((typeof(screen)=="undefined")?"":
";s"+screen.width+"*"+screen.height+"*"+(screen.colorDepth?
screen.colorDepth:screen.pixelDepth))+";u"+escape(document.URL)+
";h"+escape(document.title.substring(0,80))+
";"+Math.random();//--></script><!--/LiveInternet-->
END;
        } else {
            return $nav_menu.
<<<END
<!--LiveInternet counter--><script type="text/javascript"><!--
new Image().src = "//counter.yadro.ru/hit?r"+
escape(document.referrer)+((typeof(screen)=="undefined")?"":
";s"+screen.width+"*"+screen.height+"*"+(screen.colorDepth?
screen.colorDepth:screen.pixelDepth))+";u"+escape(document.URL)+
";h"+escape(document.title.substring(0,80))+
";"+Math.random();//--></script><!--/LiveInternet-->
END;
        }
    }
}
