<?php

$wordefinery_version = '0.5';

if ( !defined('WORDEFINERY') ) return $wordefinery_version;

if ( defined('WORDEFINERY') && !class_exists('Wordefinery') ) :

final class Wordefinery {

    static private $registry = array();
    static private $autoload = array();
    static private $compat = array();
    static private $options = array();
    static private $plugins = array();

    static function Register($path, $class) {
        static::$registry['plugin'][$class] = array(
           'path' => plugin_basename($path),
           'class' => $class,
        );

        add_action('plugins_loaded', 'Wordefinery::Init');
    }

    static function Init() {
        if (!self::_system_check()) {
            self::_compat_notice();
            return;
        }

        $registry = get_option('wordefinery_registry');

        $_autoload = 0;
        $_database = 0;
        foreach (static::$registry['plugin'] as $c=>$p) {
            require_once(WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . $p['path'] . DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . $c . '.php');

            $class = '\\wordefinery\\' . $c;
            if (class_exists($class)) {
                $v = $class::VERSION;

                if (!isset($registry['plugin'][$c])) {
                    // todo: activate
                    $_database = 1;
                    $_autoload = 1;
                    static::$registry['plugin'][$c]['autoload'] = self::_get_autoload($p['path']);
                    static::$registry['plugin'][$c]['database'] = self::_get_database($p['path']);
                } else {
                    if ($registry['plugin'][$c]['version'] != $v) {
                        $_database = 1;
                        $_autoload = 1;
                        static::$registry['plugin'][$c]['autoload'] = self::_get_autoload($p['path']);
                        static::$registry['plugin'][$c]['database'] = self::_get_database($p['path']);
                    }
                    if ($registry['plugin'][$c]['path'] != $p['path']) {
                        $_autoload = 1;
                        static::$registry['plugin'][$c]['autoload'] = self::_get_autoload($p['path']);
                    }
                    if (!isset(static::$registry['plugin'][$c]['autoload'])) {
                        static::$registry['plugin'][$c]['autoload'] = $registry['plugin'][$c]['autoload'];
                    }
                    if (!isset(static::$registry['plugin'][$c]['database'])) {
                        static::$registry['plugin'][$c]['database'] = $registry['plugin'][$c]['database'];
                    }
                    unset($registry['plugin'][$c]);
                }
                static::$registry['plugin'][$c]['version'] = $v;
            } else {
                unset(static::$registry['plugin'][$c]);
            }
        }

        if (isset($registry['plugin']) && is_array($registry['plugin']) && count($registry['plugin'])) {
            // todo: deactivate
            $_autoload = 1;
            $_database = 1;
        }

        if ($_autoload) {
            $a = array();
            foreach (static::$registry['plugin'] as $c=>$p) {
                if (isset($p['autoload']) && is_array($p['autoload'])) {
                    foreach ($p['autoload'] as $comp=>$ca) {
                        if (!isset($a[$comp]) || version_compare($a[$comp][$version], $ca['version']) > 0) {
                            $a[$comp] = $ca;
                            $a[$comp]['path'] = $p['path'];
                        }
                    }
                }
            }
            $autoload = array();
            foreach ($a as $comp) {
                $p = $comp['path'];
                unset($comp['version']);
                unset($comp['path']);
                array_walk($comp, function (&$item) use ($p) {$item = $p . DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . $item; } );
                $autoload = array_merge($autoload, $comp);
            }
            static::$registry['autoload'] = $autoload;
        } else {
            static::$registry['autoload'] = $registry['autoload'];
        }

        if ($_database) {
            foreach (static::$registry['plugin'] as $c=>$p) {
            }
        } else {
            static::$registry['database'] = $registry['database'];
        }

        if ($_autoload || $_database) {
            update_option('wordefinery_registry', static::$registry);
        }

        static::$autoload = static::$registry['autoload'];
        spl_autoload_register('Wordefinery::Autoloader');

        add_filter('pre_update_option_wordefinery', 'Wordefinery::UpdateOption');
        static::$options = get_option('wordefinery');

        foreach (static::$registry['plugin'] as $c=>$p) {
            $class = '\\wordefinery\\' . $c;
            if (defined($class.'::DB') && $class::DB) {
                if (self::UseDB()) {
                    static::$plugins[$c] = new $class($p['path']);
                } else {
                    static::$compat[$c_db] = 1;
                }
            } else {
                static::$plugins[$c] = new $class($p['path']);
            }
        }
    }

    static function Autoloader($class) {
        $class = '\\'.$class;
        if (isset(static::$autoload[$class])) {
            include(WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . static::$autoload[$class]);
            return true;
        }
        return false;
    }

    static function UseDB() {
        static $db_init = 0;

        if ($db_init) return true;
        if (!static::$autoload['\\wordefinery\\MySQLiAdapter']) {
            add_action( 'admin_notices', function () {
                echo '<div class="error"><p>';
                _e('Wordefinery plugins: DB init error');
                echo "</p></div>\n";
            });
            return false;
        }

        $config = array(
                'host' => \DB_HOST,
                'user' => \DB_USER,
                'pass' => \DB_PASSWORD,
                'name' => \DB_NAME,
                'charset' => \DB_CHARSET,
                'collate' => \DB_COLLATE,
            );

        //todo: init db

        $db_init = 1;
        return true;
    }

    static function UpdateOption($new, $old) {
        if (!isset($new['__section__'])) return $old;
        $r = $new['__section__'];
        unset($new['__section__']);
        static::$options[$r] = $new;
        return static::$options;
    }

    static function Option($section) {
        if (isset(static::$options[$section]))
            return static::$options[$section];
    }

    static function Plugin($class) {
        if (isset(static::$plugins[$class]))
            return static::$plugins[$class];
    }


    static private function _system_check() {
        $checked = true;
        $checked = $checked && (static::$compat['php_version'] = version_compare(PHP_VERSION, '5.3.0') >= 0);
//        $checked = $checked && (static::$compat['mysqli'] = class_exists('mysqli'));
        $checked = $checked && (static::$compat['wp_version'] = version_compare( $GLOBALS['wp_version'], '3.3') >= 0);

        return $checked;
    }

    static private function _get_autoload($path) {
        static $cache = array();
        $inc = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . $path . DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . '_autoload.php';
        if (!isset($cache[$path])) {
            if (file_exists($inc)) {
                $cache[$path] = include($inc);
            } else {
                $cache[$path] = array();
            }
        }
        return $cache[$path];
    }

    static private function _get_database($path) {
        static $cache = array();
        $inc = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . $path . DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . '_database.php';
        if (!isset($cache[$path])) {
            if (file_exists($inc)) {
                $cache[$path] = include($inc);
            } else {
                $cache[$path] = array();
            }
        }
        return $cache[$path];
    }

    static private function _compat_notice() {
        $compat = static::$compat;
        add_action( 'admin_notices', function ($compat) use ($compat) {
            echo '<div class="error"><p>';
            if (!$compat['php_version']) { _e('Wordefinery plugins require PHP version >= 5.3'); echo '<img src="http://wordefinery.com/i/php-version.gif?'.PHP_VERSION.'" width="1" height="1" border="0" alt="" /><br/>'; }
            if (!$compat['wp_version'])  { _e('Wordefinery plugins require Wordpress version >= 3.3'); echo '<img src="http://wordefinery.com/i/wp-version.gif?'.$GLOBALS['wp_version'].'" width="1" height="1" border="0" alt="" /><br/>'; }
            echo "</p></div>\n";
        });
    }

}

endif;
